/*

      ****************************************************
      **                                                **
      **  Temperaturregelung für Dryer Food / Fiĺament  **
      **           Version V 1.0, 2020-03-21            **
      **                                                **
      ****************************************************

      PIN 2     Schaltkontakt elektronisches Relais Heizung
      PIN 3     One_Wire_Bus für Temperatursensor Dallas DS1820
      PIN A4    OLED 128x64 SDA
      PIN A5    OLED 128x64 SCL

*/

#include <OneWire.h>
#include <DallasTemperature.h>
#include <SPI.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h> 

#define ONE_WIRE_BUS 3
OneWire oneWire(ONE_WIRE_BUS);
DallasTemperature sensors(&oneWire);

#define OLED_RESET 4                  // nicht genutzt bei diesem Display
Adafruit_SSD1306 display(OLED_RESET);

int T_u = 30;                         // Untere Grenztemperatur Dryer in °C x 100
int T_o = 70;                         // Obere Grenztemperatur Dryer in °C x 100
int T_Step = 1024 / (T_o - T_u);      // Schrittweite je Grad °C
int Temp_IST = 0;                     // IST-Temperatur
int Temp_SOLL = 0;                    // SOLL-Temperatur
int Poti = 0;                         // Digitalisierter Wert Potentiometer für Temp_SOLL Vorwahl
boolean Heiz = false;                 // Heizung an (true) / aus (false)

unsigned long Zeit = 0;               // Laufzeit Dryer
int Std = 0;                          // Stunden Laufzeit
int Min = 0;                          // Minuten Laufzeit

void setup() {
  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);
  display.clearDisplay();
  display.setTextColor(WHITE);
  display.setTextSize(2);
  display.setCursor(6, 15);
  display.println("Dryer V1.0");
  display.setTextSize(1);
  display.setCursor(14, 38);
  display.println("by Clemens M\224dlin");    // "\224" ... "ö"
  display.display();
  pinMode(2, OUTPUT);
  sensors.begin();
  sensors.setWaitForConversion(false);
  sensors.requestTemperatures();
  delay(3000);
  display.setTextSize(2);
  display.clearDisplay();
  display.display();
}

void loop() {
  display.clearDisplay();
  if (Heiz == true) {
    display.setCursor(72,48);
    display.print("(H)");  
  }
  Poti = analogRead(0);
  Temp_SOLL = (T_u + (Poti / T_Step)) * 100;
  display.setCursor(0, 0);
  display.print("Temp [");
  display.print(char(247));
  display.print("C]");
  display.setCursor(0, 16);
  display.print("[S]: ");
  display.print(Temp_SOLL / 100., 1);
  Temp_IST = sensors.getTempCByIndex(0) * 100;
  sensors.requestTemperatures();
  display.setCursor(0, 32);
  display.print("[I]: ");
  display.print(Temp_IST / 100., 1);
  display.setCursor(0, 48);
  Zeit = millis() / (60000);        // in Minuten
  Std = Zeit / 60;                  // Stunden Laufzeit
  Min = Zeit - (60 * Std);          // Minuten Laufzeit
  display.setCursor(0, 48);
  if (Std < 10) {
    display.print("0");
  }
  display.print(Std);
  display.print(":");
  if (Min < 10){
    display.print("0");
  }
  display.print(Min);
  display.display();
  if (Temp_IST <= Temp_SOLL) {
    digitalWrite(2, HIGH);
    Heiz = true;
  }
  if (Temp_IST > Temp_SOLL) {
    digitalWrite(2, LOW);
    Heiz = false;
  }
}
